#!/bin/sh

# httpd - This shell script starts/restart/stops the Apache Server

BASE=/fabos/webtools/bin
CONF=$BASE/httpd.conf
CTRL=$BASE/apachectl
HTTPD=$BASE/apachectl

RM=/bin/rm
SED=/bin/sed
MKDIR=/bin/mkdir
LOGDIR=/tmp/webtools/logs
OUTDEVICE=$LOGDIR/webtools.alert

ARGC=$#
ARG=$1
INST_ID=$2
INST_IP=$3
INST_FCIP=$4
SOCKDIR=/tmp/webtools/tmp_$INST_ID
PLATFORM=

logMessage() {
  msg=$*
	dt=`date '+%d%b%y_%H%M%S'`

	# Output to console always
  echo $msg

	# Output headers for first time
	if [ ! -f $OUTDEVICE ]; then
		echo "Webtools not available alert" >> $OUTDEVICE
		echo "----------------------------" >> $OUTDEVICE
		echo "Creation date: $dt" >> $OUTDEVICE
		echo "" >> $OUTDEVICE
		echo "Switch Instance Id -> $INST_ID" >> $OUTDEVICE
		echo "---------------------------" >> $OUTDEVICE
		echo "Reason: " >> $OUTDEVICE
		echo "------- " >> $OUTDEVICE
		echo "" >> $OUTDEVICE
	fi
	echo [$dt]: $msg >> $OUTDEVICE
}

checkPlatform()
{
  if [ ! -x /sbin/sin ] ; then
    logMessage "Unable to determine System Inventory."
    return 1 ;
  fi

	PLATFORM=`/sbin/sin | grep SWBD | cut -d ' ' -f 2 | cut -c 5-6 2>/dev/null`
	if [ "$PLATFORM" != "10" -a "$PLATFORM" != "11" -a "$PLATFORM" != "12" ]; then
		logMessage "Platform code $PLATFORM not supported."
		return 1 ;
  fi
 	
	return 0;
}

# Validate Ip Address
# Argument - ip address in dot notation
# return 0 if valid, 1 otherwise

isValidIp()
{
   ip=$1

   if [ "$ip" = "0.0.0.0" ] || 
			[ "$ip" = "255.255.255.255" ] || [ "$ip" = "" ]; then
      return 1; 
	 fi

	 return 0;

}

# Validate both eth_ip and fc_ip address
# Argument order is eth_ip and fc_ip
# return 0 if valid, 1 otherwise

checkIpAddress()
{
   ethip=$1
	 fcip=$2

	 badEthIp=0
	 badFcIp=0

	 isValidIp $ethip

	 if [ $? -ne 0 ] ; then
		 badEthIp=1
   fi

	 isValidIp $fcip

	 if [ $? -ne 0 ] ; then
		 badFcIp=1
   fi

	 if [ "$badEthIp" = "1" -a "$badFcIp" = "1" ]; then
			logMessage "Invalid IP Address configuration for Switch[$INST_ID]."
      logMessage "Eth Ip: $ethip,  FcIp: $fcip"
			return 1
   fi


	 return 0;
}

checkPaths()
{
  # This directory must be created since it is not saved in the flash
	if [ -x $MKDIR ] ; then
		$MKDIR -p $LOGDIR ;
	  chmod 777 $LOGDIR
	else 
		# Assuming tmp must exist
		OUTDEVICE=/tmp/webtools.alert
		logMessage "Unable to create $LOGDIR directory, $MKDIR not found." ;
		exit 0 ;
	fi

	if [ ! -x $SED ] ; then
		logMessage "$SED not found, http server not started." ;
		exit 0 ;
	fi

	if [ ! -x $RM ] ; then
		logMessage "Warning: $RM not found" ;
	fi

  if [ ! -d $SOCKDIR ] ; then
    $MKDIR -p $SOCKDIR
	  chmod 777 $SOCKDIR
	fi

} 


# Set up configuration files for starting multiple instances

setupConfig()
{
	#echo Initializing apache for instance[$INST_ID] with BindAddress [$INST_IP] ...

	INST_CONF=`echo $CONF'.'$INST_ID`

	if [ $ARGC -ge 4 ] ; then
    isValidIp $INST_FCIP
    if [ $? -eq 0 ] ; then
	    FC_LISTEN=`echo Listen $INST_FCIP:80` 
    else
		  FC_LISTEN=""
    fi
	fi

	isValidIp $INST_IP
  if [ $? -eq 0 ] ; then
	  ETH_LISTEN=`echo Listen $INST_IP:80` 
  else
    ETH_LISTEN="$FC_LISTEN"
    FC_LISTEN=""
	fi

# /^\#Listen/s/\#Listen [.0-9.:]*/Listen '"$INST_IP:80"'/g
  $SED -e '
    /^\#Listen/s/\#Listen [.0-9.:]*/'"$ETH_LISTEN"'/g
		/^Listen/{G;s/$/'"$FC_LISTEN"'/;G;}
    /^ErrorLog/s/error_log/error_log_'"$INST_ID"'/g
    /^CustomLog/s/access_log/access_log_'"$INST_ID"'/g
    s/httpd.lock/httpd.lock.'"$INST_ID"'/g
    s/httpd.pid/httpd.pid.'"$INST_ID"'/g
    /FastCgiIpcDir/s|^.*|FastCgiIpcDir '"$SOCKDIR"'|g
   ' $CONF > $INST_CONF
  $SED -e '
		s/httpd.conf[.0-9.]*/httpd.conf.'"$INST_ID"'/g
		s/httpd.pid[.0-9.]*/httpd.pid.'"$INST_ID"'/g
		s/FABOS_SWITCHNO=[ 0-9 ]*/FABOS_SWITCHNO='"$INST_ID"'/g
	 ' $CTRL > $CTRL.$INST_ID
  chmod +x $CTRL.$INST_ID
}

cleanup()
{
	INST_CONF=`echo $CONF'.'$INST_ID`
	INST_CTRL=`echo $CTRL'.'$INST_ID`
  INST_PID=httpd.pid.$INST_ID
  INST_LOCK=httpd.lock.$INST_ID

	if [ -f $INST_CONF ] ; then
		#echo Removing file $INST_CONF ...
		$RM $INST_CONF ;
	fi

	if [ -f $INST_CTRL ] ; then
		#echo Removing file $INST_CTRL ...
		$RM $INST_CTRL ;
	fi
	if [ -f $INST_PID ] ; then
		#echo Removing file $INST_PID ...
		$RM $INST_PID ;
	fi
	if [ -f $INST_LOCK ] ; then
		#echo Removing file $INST_LOCK ...
		$RM $INST_LOCK ;
	fi

	if [ -d $SOCKDIR ] ; then
		#echo Removing file $SOCKDIR ...
		$RM -rf $SOCKDIR ;
	fi
}

main()
{
	# bail out quietly if httpd does not exist

  if [ ! -f $HTTPD ] ; then 
		logMessage "Missing control file: $HTTPD"
		exit 1;
	fi

	checkPaths

	checkPlatform
	if [ $? -ne 0 ] ; then
    exit 1;
  fi

	if [ $ARGC -ge 3 ] ; then
	  checkIpAddress $INST_IP $INST_FCIP
	  if [ $? -ne 0 ] ; then
        exit 1
    fi
	fi

	case $ARG in

		start)  
			 # start server
	     if [ $ARGC -ge 3 ] ; then
		      setupConfig
					if [ -s $CTRL.$INST_ID ] ; then
		 				HTTPD=$CTRL.$INST_ID
					fi
  		 		if [ ! -x $HTTPD ] ; then
						logMessage "Missing control file: $HTTPD" 
						exit 0
					fi
	     fi
			 echo "starting http server [$INST_ID] ..."
			 $HTTPD start
			 echo done.
			 ;; 

		stop)
			# stop server
	    if [ $ARGC -ge 2 ] ; then
		    if [ -s $CTRL.$INST_ID ] ; then
		 	    HTTPD=$CTRL.$INST_ID
		    fi
  		 	if [ ! -x $HTTPD ] ; then
					logMessage "Missing control file: $HTTPD" 
					exit 0
				fi
	    fi
			echo "stopping http server [$INST_ID] ..."
			$HTTPD stop
			cleanup
			echo done.
			;;

		restart)
			# restart server
	    if [ $ARGC -ge 3 ] ; then
		      setupConfig
					if [ -s $CTRL.$INST_ID ] ; then
		 				HTTPD=$CTRL.$INST_ID
					fi
  		 	  if [ ! -x $HTTPD ] ; then
				  	logMessage "Missing control file: $HTTPD" 
				  	exit 0
				  fi
	    fi
			echo "restarting http server [$INST_ID] ..."
			$HTTPD restart
			echo done.
			;;

		*)
			# usage
			echo "usage: $0 <start|stop|restart> InstanceNumber EthIpAddr [FcIpAddr]"
			;;
	esac
}

main
