#!/bin/sh
#
#
#    Copyright (c) 1996-2001 Brocade Communications Systems, Inc.
#    All rights reserved.
#
#    File name:   switchess
#    Module name: diag/scripts
#
#    This file operates the ess activity.
#
# NAME
#    ess - Supports three activities
#    1. Set ess command diagsetsycle for ess
#    2. Containes the ess activity list
#    3. Processes the ess activity
#
# SYNOPSIS
#    switchess [ diagsetcycle ]
#
    # AVAILABILITY
#    swt_mfg
#
# DESCRIPTION
#    This command will manage the ess activity. It contains and
#    manages the parameters needed this activity. It also cointains
#    the list of activities to perform during a single ess run. 
#    It has the processing loop to execute the commands in the list.
#
#    The mode becomes active as soon as this command is executed.
#    It does not require a reboot to take effect.
#
# OPERANDS
#
#    "diagsetcycle"    This operand will trigger activity related to 
#                      editing the avialable parameters active during
#                      the ess activity. The user will be prompted 
#                      to modify each parameter in the odrer specified 
#                      in the this script. The current value, default 
#                      value, and description of the parameter are 
#                      displayed. If the user does not want to modify 
#                      the value, then a <cr> is recognized as 
#                      "hold current value". Any other value for this
#                      parameter is ignored.
#
################################
# The function create_param_file() is required in this script.
# It is intended to be modified to define the persistent 
# parameters required by the script. It must only be modified
# between the "#Make changes after this line" and 
# "#Make changes above this line".
#
# All entries starting with PARAMDEF will be available in 
# diagsetcycle ess command. Lines beginning with "PARAMDEF"
# will define parameters that will persistent across power 
# cycles and are available as environment variables during the
# duration of the script.
#
# A PARAMDEF line can not contain leading spaces. The parameters 
# will appear for editing in the same order as they are listed.
# Each PARAMDEF entry contain several parts seperated by a 
# single space character. The information is interputed in a 
# specific order. The parts are:
#   1. keyword (PARAMDEF) No leading spaces are allowed.
#   2. variable name (single word) space seperated
#   3. default value for parameter
#   4. type of variable ($STRING, $INTEGER)
#   5. minimum value (only for integer, NA if not applicable)
#   6. maximum value (only for integer, NA if not applicable)
#   7. description (rest of the line until "\n")
#
# Any line not starting with PARAMDEF is considered a comment, and 
# is not processed.
#
# The following is an example of declaring a variable to use 
# in diagsetcycle to control the -nmegs value for crossPortTest.
# It is then used in the command definition as a variable derived 
# from the shell.
# Example:
#
#  ------------------------------------- 1 definition
#  |        ---------------------------- 2 variable name
#  |        |     ---------------------- 3 default value
#  |        |     |   ------------------ 4 type
#  |        |     |   |        --------- 5 min
#  |        |     |   |        | ------- 6 max
#  |        |     |   |        | |  ---- 7 comment
#  |        |     |   |        | |  |
#  |        |     |   |        | |  |
#  |        |     |   |        | |  |
#  v        v     v   v        v v  v
#  PARAMDEF nmegs 150 $INTEGER 0 NA Million frames for crossPortTest
#          crossporttest -nmegs \$nmegs -lb_mode 1
#
##################################

create_param_file()
{
/bin/cat << EOF
#Make changes after this line
PARAMDEF number_of_runs 0 $INTEGER -1 NA number of passes of ess (0=infinite)
PARAMDEF vib 2 $INTEGER 0 NA number of vibration minutes
PARAMDEF thermal 10 $INTEGER 0 30 number of thermal cycles
PARAMDEF label HALT $STRING NA NA Label for ess start and stop messages
PARAMDEF margin_on  1 $INTEGER 0 4 0-none, 1-(HIGH,NOMINAL,LOW), 2-HIGH, 3-LOW, 4-NOMINAL
PARAMDEF min_lb_mode  1 $INTEGER 1 5 Limits -lb_mode of tests
PARAMDEF tbr_passes 1 $INTEGER 0 NA turboRamTest number of passes
PARAMDEF prt_on  0 $INTEGER 0 1 Enable portregtest (0=off,1=ON)
PARAMDEF cntmem_on  0 $INTEGER 0 1 Enable centralmemorytest (0=off,1=ON)
PARAMDEF cmi_on  1 $INTEGER 0 1 Enable cmitest (0=off,1=ON)
PARAMDEF retention_on  0 $INTEGER 0 1 Enable SRAM/CMEM retention tests (0=off,1=ON)
PARAMDEF cam_on  0 $INTEGER 0 1 Enable camtest (0=off,1=ON)
PARAMDEF flt_passes 0 $INTEGER 0 NA filtertest number of passes
PARAMDEF sta_passes 0 $INTEGER 0 NA statstest number of passes
PARAMDEF plb_nframes 0 $INTEGER 0 NA portLoopbackTest number of frames
PARAMDEF txd_nframes 0 $INTEGER 0 NA txdpath number of frames
PARAMDEF xpt_nframes 5 $INTEGER 0 NA crossPortTest number of frames
PARAMDEF bpt_nframes 0 $INTEGER 0 NA backportTest number of frames
PARAMDEF slk_nmegs 0 $INTEGER 0 NA spinSilkTest number of million frames
PARAMDEF bpt_all_nframes 10 $INTEGER 0 NA (all blades) backportTest number of frames
PARAMDEF slk_all_nmegs 10 $INTEGER 0 NA (all blades) spinSilkTest number of million frames
#Make changes above this line
EOF
}

##################################
# The function create_command_file() is required in this script.
# It is the list of commands to execute during each ess pass.
# The commands will be executed in order from top to bottom.  It
# must only be modified between the "#Make changes after this line"
# and "#Make changes above this line".
#
# A command is defined as any command line string that can be
# executed from the telnet prompt preceeded by a indicator of if the
# command is enabled. These include fabos commands, scripts,
# diagnostics, and internal functions within this script.
#
# Example:
#
#  ------------------------------------- 1 enable (0=disable)
#  |          -------------------------- 2 command
#  |          |             ------------ 3 arguments
#  |          |             |
#  |          |             |
#  |          |             |
#  v          v             v
#  \$margin_on voltageMargin -margin \$margin
#
# The enable can be hardcoded "0" (off) or (1) ON, or it can be a
# PARAMDEF. The paramdef is used in the example.
#
# The return status of the command is included in the overall 
# status of the the ess pass.  The script assumes that a 
# command exit status of 0 indicates PASS, and any other value
# is a fail. If the commands do not return values in this way,
# then awrapper function in this script must be added that serves 
# to translate the command exit status in the the 0=PASS, !0=FAIL
# status. This wrapper function would be placed in the command list.
#
# The PARAMDEF names can be used in place of arguments to the 
# commands (-lb_mode, -nmegs, ...). They can also be used for
# any variable within the run processing. For example the first
# one in the list can be the number of ess passes to complete,
# or on which pass to modify the voltage margining.
#
# Regular shell variables can also be used, however they are not 
# maintained across power cycles with the exception of $RunNumber,
# $RunStatus $CumulativeStatus $State. These are used by the 
# ess script and are available for use.
#
# CommandNumber    - indicates the line of command file being processed.
#                    Incremented during runBurninCommand()
# RunNumber        - the number of ess passes activated.
# RunStatus        - 0/1 PASS/FAIL status of active run.
# CumulativeStatus - 0/1 PASS/FAIL status of the all active runs.
# NumberOfRuns     - number of ess passes to complete. (PARAMDEF)
# State            - ACTIVE (continue running ess)
#                    COMPLETE (ess not started)
# LogFile          - Path and file containing entire log of command and shell output
# UserFile         - File name for this scripts purpose
#
# (NOTE: UserFile is only a file name, and does not contain a path. If it is
#  to be used in this script, it needs path set local to this script.)
#
# The entire path to the Linux commands must be specified. This is due
# to the context inwhich this script executes. This is not required for 
# shell internals like eval. It is needed for expr. Please type 
# which <command> to get the path to the command.
#
# When using variable references in a command argument, the "$"
# in the reference must be proceeded by a "\" to prevent the 
# shell from inturputing the "$". Also make sure the referenced variable
# is exported. The export of PARAMDEF variables is automatic.
##################################

create_command_file()
{
/bin/cat << EOF
#Make changes after this line
# Set the +5V to the high margin, and wait for power to settle
\$margin_on voltageMargin -margin \$margin
\$margin_on /bin/sleep 3

# Turboram test checks ASIC SRAMS
\$tbr_passes turboramtest -passcnt \$tbr_passes --gettemp 1

# Portregtest checks rest of ASIC PCI.
\$prt_on portregtest --gettemp 1

# Check central memory RAM
\$cntmem_on centralmemorytest --gettemp 1

# Check Command Message Interface
\$cmi_on cmitest --gettemp 1

# SRAM retention tests
\$retention_on sramretentiontest --gettemp 1
\$retention_on cmemretentiontest --gettemp 1

# ASIC Cam / filter / statistics
\$cam_on camtest --gettemp 1
\$flt_passes filtertest -passcnt \$flt_passes -txports \$UserPorts --gettemp 1
\$sta_passes statstest -passcnt \$sta_passes -ports \$UserPorts --gettemp 1

# Portloopbacktest internal loopback 2 GB/S
\$plb_nframes portloopbacktest -lb_mode 5 -nframes \$plb_nframes -spd_mode 2 --gettemp 1

# Portloopbacktest serdes loopback 1 GB/S
\$plb_nframes portloopbacktest -lb_mode \$ptlb_lb_mode -nframes \$plb_nframes -spd_mode 1 --gettemp 1

# Portloopbacktest serdes loopback 2 GB/S
\$plb_nframes portloopbacktest -lb_mode \$ptlb_lb_mode -nframes \$plb_nframes -spd_mode 2 --gettemp 1

# txdpath connections on current blade
\$txd_nframes txdpath -nframes \$txd_nframes --gettemp 1

# Crossporttest with SFP / loopback plugs  1 GB/S
\$xpt_nframes crossporttest -nframes \$xpt_nframes -lb_mode \$xpt_lb_mode -spd_mode 1 -ports \$UserPorts --gettemp 1

# Crossporttest with SFP / loopback plugs  2 GB/S
\$xpt_nframes crossporttest -nframes \$xpt_nframes -lb_mode \$xpt_lb_mode -spd_mode 2 -ports \$UserPorts --gettemp 1

# Backport Miniswitch - Miniswitch connections on current blade
\$bpt_nframes backport -nframes \$bpt_nframes -ports \$UserPorts --gettemp 1

# Spinsilk per-blade internal 2 GB/S
\$slk_nmegs spinsilk -nmegs \$slk_nmegs -lb_mode 5 -spd_mode 2 -ports \$UserPorts --gettemp 1

# Spinsilk per-blade Underrun + Serdes tests Even 2 GB/S Odd 1 GB/S
\$slk_nmegs spinsilk -nmegs \$slk_nmegs -lb_mode \$slkser_lb_mode -spd_mode 5 -ports \$UserPorts --gettemp 1

# Spinsilk per-blade Underrun + Serdes tests Even 1 GB/S Odd 2 GB/S
\$slk_nmegs spinsilk -nmegs \$slk_nmegs -lb_mode \$slkser_lb_mode -spd_mode 6 -ports \$UserPorts --gettemp 1

# Spinsilk per-blade Cable 2 GB/S
\$slk_nmegs spinsilk -nmegs \$slk_nmegs -lb_mode \$slkcbl_lb_mode -spd_mode 2 -ports \$UserPorts --gettemp 1

#
# The following two commands run on the full system.  They must
# be removed for strictly per-slot testing.
#

# Backport all ports in full system 
# See BEFORE_RUN for conversion of bpt_ALL_nframes from bpt_all_nframes
\$bpt_ALL_nframes backport -nframes \$bpt_ALL_nframes -extonly 1 --gettemp 1

# Spinsilk full speed all ports in system
# See BEFORE_RUN for conversion of slk_ALL_nmegs from slk_all_nmegs
\$slk_ALL_nmegs spinsilk -nmegs \$slk_ALL_nmegs -lb_mode \$slkcbl_lb_mode --gettemp 1

#Make changes above this line
EOF
}

##################################
# Example functions to demonstrate ability to modify things 
# at two critical places in the activity:
#
# 1. beginning of a run
# 2. end of a run
#
# At both places, environment variable can be used
# (read or write) in the context of the function.
#
# Since the PARAMDEF's are saved to the environment, they are
# also available to the functions. These functions can call
# the xtool scripts, or other commands and functions in the script.
##################################
export margin=INITIAL

BEFORE_RUN()
{
  # Adjust margining to cycle every three runs
  case $margin_on in
  "0" )
    volt_state="?";;
  "1" )
    _index=`/usr/bin/expr $RunNumber % 3`
    case $_index in
    "0") export margin=HIGH;volt_state=H;;
    "1") export margin=NOMINAL;volt_state=N;;
    "2") export margin=LOW;volt_state=L;;
    *) export margin=UNKNOWN;volt_state=?;;
    esac
    burninMessage "BEFORE_RUN: Setting margin $margin";;
  "2" )
    volt_state=H
    margin=HIGH
    voltageMargin -margin HIGH | /usr/bin/tee -a $LogFile;;
  "3" )
    volt_state=L
    margin=LOW
    voltageMargin -margin LOW | /usr/bin/tee -a $LogFile;;
  "4" )
    volt_state=N
    export margin=NOMINAL
    voltageMargin -margin NOMINAL | /usr/bin/tee -a $LogFile;;
  esac

  export DIAG_PKG_STR="1$volt_state"

  # Adjust for the system level tests to run when there is a slot match
  bpt_ALL_nframes=$bpt_all_nframes
  slk_ALL_nmegs=$slk_all_nmegs
  #
  # Adjust the lb_mode values based in min_lb_mode
  #
  xpt_lb_mode=1
  ptlb_lb_mode=2
  slkser_lb_mode=2
  mnc_lb_mode=1
  slkcbl_lb_mode=1

  if [ $xpt_lb_mode -lt $min_lb_mode ]
  then
    burninMessage "BEFORE_RUN: Limiting lb_mode to $min_lb_mode for crossporttest."
    xpt_lb_mode=$min_lb_mode
  fi

  if [ $ptlb_lb_mode -lt $min_lb_mode ]
  then
    burninMessage "BEFORE_RUN: Limiting lb_mode to $min_lb_mode portloopbacktest."
    ptlb_lb_mode=$min_lb_mode
  fi

  if [ $slkser_lb_mode -lt $min_lb_mode ]
  then
    burninMessage "BEFORE_RUN: Limiting lb_mode to $min_lb_mode spinsilk Serdes."
    slkser_lb_mode=$min_lb_mode
  fi

  if [ $mnc_lb_mode -lt $min_lb_mode ]
  then
    burninMessage "BEFORE_RUN: Limiting lb_mode to $min_lb_mode minicycle."
    mnc_lb_mode=$min_lb_mode
  fi

  if [ $slkcbl_lb_mode -lt $min_lb_mode ]
  then
    burninMessage "BEFORE_RUN: Limiting lb_mode to $min_lb_mode spinsilk Cable."
    slkcbl_lb_mode=$min_lb_mode
  fi
}

AFTER_RUN()
{
  burninMessage "AFTER_RUN: run status: $RunStatus run `/usr/bin/expr $RunNumber - 1` cumulative $CumulativeStatus"
}

##################################
# Example function to demonstrate ability to do things 
# during the command list
#
# Environment variable can be read, but modifications will not
# propigate beyond the context of this function. They could be 
# saved in a file in /tmp, and then read in by a different 
# function later in the processing, or by the BEFORE_RUN or 
# AFTER_RUN functions. If items are to be saved in a file, please use 
# the (path)$UserFile.* name for the file. Where * can be anything
# you want. Do not forget to cleanup these files. Since the script can 
# be aborted, a function my_local_cleanup should include any such
# activity.
#
# (NOTE: UserFile is only a file name, and does not contain a path. If it is
#  to be used in this script, it needs path set local to this script.)
#
# Since the PARAMDEF's are saved to the environment, they are
# also available to the functions. These functions can call
# the xtool scripts, or other commands and functions in the script.
##################################
my_local_cleanup() 
{
  burninMessage "Done with cleanup"
}

##################################
# Main body of script
# DIAGSETCYCLE_CAPABLE
##################################
# Load common functions
# Should be first to ensure the library 
# functions are available for use
##################################
home="/fabos/share"
util="diagcommon.sh"
ok=0
for f in "./$util" "$FABOSHOME/share/$util" "$home/$util" ; do
	if [ -r $f ] ; then
		. $f
		ok=1
		break;
	fi
done
if [ $ok -ne 1 ] ; then
	err "Could not locate $util"
	exit 3
fi

#
# get sys info for multi blade
#
multiBlade=`isMultiBlade`
if [ $multiBlade = TRUE ]
then
     export syntax="$0 -slot [slot] essScript"
else
     export syntax="$0 essScript"
fi

##################################
# Check if it is OK to begin the run ess
##################################
checkBurninState $*

##################################
# MAIN PROCCESS LOOP for the commands
##################################
if [ $NumberOfRuns -eq 0 ]
then
  while [ 1 ]
  do

    ###################################
    # initialize run and indicate which pass we are beginning
    ###################################
    initializePerRunVariables

    BEFORE_RUN

    ###################################
    # Begin processing the command file
    ###################################  
    while [ "$CmdNumber" -le "$NumberOfCommands" ]
    do

      runBurninCommand

    done  
    ###################################
    # Update the status at the end of the pass
    ###################################
    burninStatusUpdate

    AFTER_RUN

  done
else
  while [ "$RunNumber" -le "$NumberOfRuns" ]
  do

    ###################################
    # initialize run and indicate which pass we are beginning
    ###################################
    initializePerRunVariables

    BEFORE_RUN

    ###################################
    # Begin processing the command file
    ###################################  
    while [ "$CmdNumber" -le "$NumberOfCommands" ]
    do

      runBurninCommand

    done  
    ###################################
    # Update the status at the end of the pass
    ###################################
    burninStatusUpdate

    AFTER_RUN

  done
fi

State=COMPLETED

cleanupOnExit

exit $CumulativeStatus
